
#include "math.h"
#include <sys/time.h>

const int N = 100 ;
const int NITER = 10000 ;

int main(int argc, char** argv) {

  int i, j ;

  float h = 1.0 / (N - 1) ;
  
  // Initialization:
  //    sol = exact solution
  //    f   = forcing term
  //    u   = approximation solution
  
  float u [N] [N], f [N] [N], sol [N] [N] ;
  
  const float fac = 4.0 - 2.0 * (cos(h) + cos(3 * h)) ;
  
  for(i = 0 ; i < N ; i++)
    for(j = 0 ; j < N ; j++) {
      sol [i] [j] = sin(3 * (h * i) + h * j) ;
      if(i == 0 || i == N - 1 || j == 0 || j == N - 1)
        u [i] [j] = sol [i] [j] ;
      else {
        u [i] [j] = 0.0 ;
        f [i] [j] = fac * sol [i] [j] ;
      }
    }

//  printf("initial u:\n") ;
//  for(i = 0 ; i < N ; i++) {
//    for(j = 0 ; j < N ; j++)
//      printf("%7.3f", u [i] [j]) ;
//    printf("\n") ;
//  }

  clock() ;

  // Main loop

  for(int iter = 0 ; iter < NITER ; iter++) {
    for(int parity = 0 ; parity < 2 ; parity++)
      for(int i1 = 1 ; i1 < N - 1 ; i1++)
	for(int j1 = 1 + (i1 + parity) % 2 ; j1 < N - 1 ; j1 += 2)
	  u [i1] [j1] = 0.25 * (f [i1] [j1] +
				u [i1 + 1] [j1] + u [i1 - 1] [j1] +
				u [i1] [j1 + 1] + u [i1] [j1 - 1]) ;

//    printf("iteration = %d\n", iter) ;
//    for(i = 0 ; i < N ; i++) {
//      for(j = 0 ; j < N ; j++)
//	printf("%7.3f", u [i] [j]) ;
//      printf("\n") ;
//    }
  }

  clock_t t = clock() ;
  
  printf("elapsed time = %f\n", (float) t / CLOCKS_PER_SEC) ;

//  printf("exact solution:\n") ;
//  for(i = 0 ; i < N ; i++) {
//    for(j = 0 ; j < N ; j++)
//      printf("%7.3f", sol [i] [j]) ;
//    printf("\n") ;
//  }
}

